"""Command Line Interface for DotDropboxIgnore"""

# Standard library imports
import argparse
import itertools
from importlib import resources
from pathlib import Path

# DotDropboxIgnore imports
from dropboxignore.paths_to_ignore import get_paths_to_ignore, read_ignore_file
from dropboxignore.shell import init_shell


def cli():
    """CLI entry point"""
    args = get_args()
    globs = read_ignore_file(Path(args.file))
    parsed_paths = get_paths_to_ignore(Path(args.path), globs)

    print("PATHS TO IGNORE:\n")
    for path in parsed_paths:
        print(f"  - {path.relative_to(Path.cwd())}")
    print()

    if ask_to_proceed():
        shell = init_shell()
        shell.ignore_folders(parsed_paths)

    raise SystemExit


def get_args():
    """Get the command line arguments"""
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "path",
        help="Path to ignore (default working directory)",
        default=Path.cwd(),
        nargs="?",
    )
    parser.add_argument(
        "-f",
        "--file",
        help="The specific ignore file (has own default)",
        default=get_ignore_file(),
    )
    return parser.parse_args()


def get_ignore_file():
    """Get the ignore file in the CWD or uses the package default"""
    return next(
        file
        for file in itertools.chain(
            Path.cwd().iterdir(), resources.files(__package__).iterdir()
        )
        if file.name == ".dropboxignore"
    )


def ask_to_proceed():
    """Asks user to proceed with operation"""
    while True:
        user_input = input("proceed? y/n\n")

        if user_input in "yY":
            return True
        elif user_input in "nN":
            return False
        else:
            print("invalid input")



"""Handles reading ignore files and generating a list of paths based on them
"""

# Standard library imports
from pathlib import Path


def read_ignore_file(path: Path):
    """Parse an ignore file of glob patterns"""
    return [
        stripped
        for line in path.read_text(encoding="UTF-8").strip().split("\n")
        if (stripped := line.strip())
    ]


def get_paths_to_ignore(root: Path, globs: list[str]):
    """Provide list of files that match any of the glob patterns"""
    paths_to_ignore = []
    for glob_pattern in globs:
        folders = root.glob(glob_pattern)

        for folder in folders:
            skip = any(
                ignored_path in folder.parents for ignored_path in paths_to_ignore
            )
            if skip:
                continue
            else:
                paths_to_ignore.append(folder)
    return paths_to_ignore



"""Defines the different shells required for ignore operations
"""

# Standard library imports
import platform
import textwrap
from abc import ABC, abstractmethod
from pathlib import Path
from subprocess import DEVNULL, run


def init_shell():
    """Detect OS and initlialize appropriate shell"""
    print("initializing shell")
    system = platform.system()
    print(f"{system} detected")
    if system == "Linux":
        return Bash_shell()
    elif system == "Windows":
        return Pwsh_shell()
    elif system == "Darwin":
        raise Bash_shell()


class Shell(ABC):
    """Interface for shells"""

    @abstractmethod
    def ignore_folders(self, paths: list[Path]):
        """Should get Dropbox to ignore all paths provided"""
        pass


class Pwsh_shell(Shell):
    """Powershell Core ignore runner"""

    def __init__(self):
        try:
            run(["pwsh", "-V"], stdout=DEVNULL, stderr=DEVNULL)
            self.shell = "pwsh"
        except FileNotFoundError:
            print("Powershell Core not installed, falling back to Windows PowerShell")
            self.shell = "powershell"

    @staticmethod
    def _make_string_path_list(paths: list[Path]):
        """Joins list of paths into one long string to pass into pwsh"""
        return "', '".join([str(path).replace("'", "`'") for path in paths])

    def ignore_folders(self, paths: list[Path]):
        """Sends ignore command"""
        path_list = self._make_string_path_list(paths)
        command = (
            f"Set-Content -Path '{path_list}' -Stream com.dropbox.ignored -Value 1"
        )
        run([self.shell, "-NoProfile", "-Command", command], check=True)
        print("Done!")


class Bash_shell(Shell):
    @staticmethod
    def _make_string_path_list(paths: list[Path]):
        """Joins list of paths into one long string to pass into bash"""
        return "' '".join([str(path).replace("'", "\\'") for path in paths])

    def get_ignored_status(self, paths: list[Path]):
        """Query the Dropbox ignored status"""
        path_list = self._make_string_path_list(paths)

        command = textwrap.dedent(
            f"""\
            for f in '{path_list}'
            do
                if  (attr -q -g com.dropbox.ignored $f)
                then
                    echo "$f"
                fi
            done"""
        )
        run(["sh", "-c", command])

    def ignore_folders(self, paths: list[Path]):
        """Sends the ignore command to bash"""
        path_list = self._make_string_path_list(paths)
        command = (
            f"for f in '{path_list}'\ndo\nattr -s com.dropbox.ignored -V 1 $f\ndone"
        )
        run(["bash", "-c", command], check=True)
        print("Done!")
